# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DCMEDIT_METADATA = Metadata(
    id="8755d3035d88c63d3618eea7e9e40463fa8a47ee.boutiques",
    name="dcmedit",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DcmeditTagParamsDictNoTag = typing.TypedDict('_DcmeditTagParamsDictNoTag', {
    "group": str,
    "element": str,
    "newvalue": str,
})
DcmeditTagParamsDictTagged = typing.TypedDict('DcmeditTagParamsDictTagged', {
    "@type": typing.Literal["tag"],
    "group": str,
    "element": str,
    "newvalue": str,
})
DcmeditTagParamsDict = _DcmeditTagParamsDictNoTag | DcmeditTagParamsDictTagged


_DcmeditConfigParamsDictNoTag = typing.TypedDict('_DcmeditConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DcmeditConfigParamsDictTagged = typing.TypedDict('DcmeditConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DcmeditConfigParamsDict = _DcmeditConfigParamsDictNoTag | DcmeditConfigParamsDictTagged


_DcmeditParamsDictNoTag = typing.TypedDict('_DcmeditParamsDictNoTag', {
    "anonymise": bool,
    "id": typing.NotRequired[str | None],
    "tag": typing.NotRequired[list[DcmeditTagParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DcmeditConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "file": InputPathType,
})
DcmeditParamsDictTagged = typing.TypedDict('DcmeditParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dcmedit"],
    "anonymise": bool,
    "id": typing.NotRequired[str | None],
    "tag": typing.NotRequired[list[DcmeditTagParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DcmeditConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "file": InputPathType,
})
DcmeditParamsDict = _DcmeditParamsDictNoTag | DcmeditParamsDictTagged


def dcmedit_tag(
    group: str,
    element: str,
    newvalue: str,
) -> DcmeditTagParamsDictTagged:
    """
    Build parameters.
    
    Args:
        group: replace specific tag.
        element: replace specific tag.
        newvalue: replace specific tag.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "tag",
        "group": group,
        "element": element,
        "newvalue": newvalue,
    }
    return params


def dcmedit_tag_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcmeditTagParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("group", None) is None:
        raise StyxValidationError("`group` must not be None")
    if not isinstance(params["group"], str):
        raise StyxValidationError(f'`group` has the wrong type: Received `{type(params.get("group", None))}` expected `str`')
    if params.get("element", None) is None:
        raise StyxValidationError("`element` must not be None")
    if not isinstance(params["element"], str):
        raise StyxValidationError(f'`element` has the wrong type: Received `{type(params.get("element", None))}` expected `str`')
    if params.get("newvalue", None) is None:
        raise StyxValidationError("`newvalue` must not be None")
    if not isinstance(params["newvalue"], str):
        raise StyxValidationError(f'`newvalue` has the wrong type: Received `{type(params.get("newvalue", None))}` expected `str`')


def dcmedit_tag_cargs(
    params: DcmeditTagParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-tag")
    cargs.append(params.get("group", None))
    cargs.append(params.get("element", None))
    cargs.append(params.get("newvalue", None))
    return cargs


def dcmedit_config(
    key: str,
    value: str,
) -> DcmeditConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dcmedit_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcmeditConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dcmedit_config_cargs(
    params: DcmeditConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DcmeditOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DcmeditParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dcmedit_params(
    file: InputPathType,
    anonymise: bool = False,
    id_: str | None = None,
    tag: list[DcmeditTagParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DcmeditConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DcmeditParamsDictTagged:
    """
    Build parameters.
    
    Args:
        file: the DICOM file to be edited.
        anonymise: remove any identifiable information, by replacing the\
            following tags:\
            - any tag with Value Representation PN will be replaced with\
            'anonymous'\
            - tag (0010,0030) PatientBirthDate will be replaced with an empty\
            string\
            WARNING: there is no guarantee that this command will remove all\
            identiable information, since such information may be contained in\
            any number of private vendor-specific tags. You will need to\
            double-check the results independently if you need to ensure\
            anonymity.
        id_: replace all ID tags with string supplied. This consists of tags\
            (0010, 0020) PatientID and (0010, 1000) OtherPatientIDs.
        tag: replace specific tag.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dcmedit",
        "anonymise": anonymise,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "file": file,
    }
    if id_ is not None:
        params["id"] = id_
    if tag is not None:
        params["tag"] = tag
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dcmedit_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcmeditParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("anonymise", False) is None:
        raise StyxValidationError("`anonymise` must not be None")
    if not isinstance(params["anonymise"], bool):
        raise StyxValidationError(f'`anonymise` has the wrong type: Received `{type(params.get("anonymise", False))}` expected `bool`')
    if params.get("id", None) is not None:
        if not isinstance(params["id"], str):
            raise StyxValidationError(f'`id` has the wrong type: Received `{type(params.get("id", None))}` expected `str | None`')
    if params.get("tag", None) is not None:
        if not isinstance(params["tag"], list):
            raise StyxValidationError(f'`tag` has the wrong type: Received `{type(params.get("tag", None))}` expected `list[DcmeditTagParamsDict] | None`')
        for e in params["tag"]:
            dcmedit_tag_validate(e)
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DcmeditConfigParamsDict] | None`')
        for e in params["config"]:
            dcmedit_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("file", None) is None:
        raise StyxValidationError("`file` must not be None")
    if not isinstance(params["file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `InputPathType`')


def dcmedit_cargs(
    params: DcmeditParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dcmedit")
    if params.get("anonymise", False):
        cargs.append("-anonymise")
    if params.get("id", None) is not None:
        cargs.extend([
            "-id",
            params.get("id", None)
        ])
    if params.get("tag", None) is not None:
        cargs.extend([a for c in [dcmedit_tag_cargs(s, execution) for s in params.get("tag", None)] for a in c])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dcmedit_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("file", None)))
    return cargs


def dcmedit_outputs(
    params: DcmeditParamsDict,
    execution: Execution,
) -> DcmeditOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DcmeditOutputs(
        root=execution.output_file("."),
    )
    return ret


def dcmedit_execute(
    params: DcmeditParamsDict,
    runner: Runner | None = None,
) -> DcmeditOutputs:
    """
    dcmedit
    
    Edit DICOM file in-place.
    
    Note that this command simply replaces the existing values without modifying
    the DICOM structure in any way. Replacement text will be truncated if it is
    too long to fit inside the existing tag.
    
    WARNING: this command will modify existing data! It is recommended to run
    this command on a copy of the original data set to avoid loss of data.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DcmeditOutputs`).
    """
    dcmedit_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DCMEDIT_METADATA)
    params = execution.params(params)
    cargs = dcmedit_cargs(params, execution)
    ret = dcmedit_outputs(params, execution)
    execution.run(cargs)
    return ret


def dcmedit(
    file: InputPathType,
    anonymise: bool = False,
    id_: str | None = None,
    tag: list[DcmeditTagParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DcmeditConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DcmeditOutputs:
    """
    dcmedit
    
    Edit DICOM file in-place.
    
    Note that this command simply replaces the existing values without modifying
    the DICOM structure in any way. Replacement text will be truncated if it is
    too long to fit inside the existing tag.
    
    WARNING: this command will modify existing data! It is recommended to run
    this command on a copy of the original data set to avoid loss of data.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        file: the DICOM file to be edited.
        anonymise: remove any identifiable information, by replacing the\
            following tags:\
            - any tag with Value Representation PN will be replaced with\
            'anonymous'\
            - tag (0010,0030) PatientBirthDate will be replaced with an empty\
            string\
            WARNING: there is no guarantee that this command will remove all\
            identiable information, since such information may be contained in\
            any number of private vendor-specific tags. You will need to\
            double-check the results independently if you need to ensure\
            anonymity.
        id_: replace all ID tags with string supplied. This consists of tags\
            (0010, 0020) PatientID and (0010, 1000) OtherPatientIDs.
        tag: replace specific tag.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DcmeditOutputs`).
    """
    params = dcmedit_params(
        anonymise=anonymise,
        id_=id_,
        tag=tag,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        file=file,
    )
    return dcmedit_execute(params, runner)


__all__ = [
    "DCMEDIT_METADATA",
    "DcmeditConfigParamsDict",
    "DcmeditConfigParamsDictTagged",
    "DcmeditOutputs",
    "DcmeditParamsDict",
    "DcmeditParamsDictTagged",
    "DcmeditTagParamsDict",
    "DcmeditTagParamsDictTagged",
    "dcmedit",
    "dcmedit_config",
    "dcmedit_execute",
    "dcmedit_params",
    "dcmedit_tag",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FOD2FIXEL_METADATA = Metadata(
    id="789d8f209b3fdf4c5f81cd43bcf2cd1ab6e4addc.boutiques",
    name="fod2fixel",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Fod2fixelConfigParamsDictNoTag = typing.TypedDict('_Fod2fixelConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Fod2fixelConfigParamsDictTagged = typing.TypedDict('Fod2fixelConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Fod2fixelConfigParamsDict = _Fod2fixelConfigParamsDictNoTag | Fod2fixelConfigParamsDictTagged


_Fod2fixelParamsDictNoTag = typing.TypedDict('_Fod2fixelParamsDictNoTag', {
    "afd": typing.NotRequired[str | None],
    "peak_amp": typing.NotRequired[str | None],
    "disp": typing.NotRequired[str | None],
    "fmls_integral": typing.NotRequired[float | None],
    "fmls_peak_value": typing.NotRequired[float | None],
    "fmls_no_thresholds": bool,
    "fmls_lobe_merge_ratio": typing.NotRequired[float | None],
    "mask": typing.NotRequired[InputPathType | None],
    "maxnum": typing.NotRequired[int | None],
    "nii": bool,
    "dirpeak": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fod2fixelConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fod": InputPathType,
    "fixel_directory": str,
})
Fod2fixelParamsDictTagged = typing.TypedDict('Fod2fixelParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fod2fixel"],
    "afd": typing.NotRequired[str | None],
    "peak_amp": typing.NotRequired[str | None],
    "disp": typing.NotRequired[str | None],
    "fmls_integral": typing.NotRequired[float | None],
    "fmls_peak_value": typing.NotRequired[float | None],
    "fmls_no_thresholds": bool,
    "fmls_lobe_merge_ratio": typing.NotRequired[float | None],
    "mask": typing.NotRequired[InputPathType | None],
    "maxnum": typing.NotRequired[int | None],
    "nii": bool,
    "dirpeak": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fod2fixelConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fod": InputPathType,
    "fixel_directory": str,
})
Fod2fixelParamsDict = _Fod2fixelParamsDictNoTag | Fod2fixelParamsDictTagged


def fod2fixel_config(
    key: str,
    value: str,
) -> Fod2fixelConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fod2fixel_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fod2fixelConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fod2fixel_config_cargs(
    params: Fod2fixelConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Fod2fixelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Fod2fixelParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    fixel_directory: OutputPathType
    """the output fixel directory"""
    afd: OutputPathType | None
    """output the total Apparent Fibre Density per fixel (integral of FOD lobe)
    """
    peak_amp: OutputPathType | None
    """output the amplitude of the FOD at the maximal peak per fixel """
    disp: OutputPathType | None
    """output a measure of dispersion per fixel as the ratio between FOD lobe
    integral and maximal peak amplitude """


def fod2fixel_params(
    fod: InputPathType,
    fixel_directory: str,
    afd: str | None = None,
    peak_amp: str | None = None,
    disp: str | None = None,
    fmls_integral: float | None = None,
    fmls_peak_value: float | None = None,
    fmls_no_thresholds: bool = False,
    fmls_lobe_merge_ratio: float | None = None,
    mask: InputPathType | None = None,
    maxnum: int | None = None,
    nii: bool = False,
    dirpeak: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fod2fixelConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Fod2fixelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fod: the input fod image.
        fixel_directory: the output fixel directory.
        afd: output the total Apparent Fibre Density per fixel (integral of FOD\
            lobe).
        peak_amp: output the amplitude of the FOD at the maximal peak per fixel.
        disp: output a measure of dispersion per fixel as the ratio between FOD\
            lobe integral and maximal peak amplitude.
        fmls_integral: threshold absolute numerical integral of positive FOD\
            lobes. Any lobe for which the integral is smaller than this threshold\
            will be discarded. Default: 0.
        fmls_peak_value: threshold peak amplitude of positive FOD lobes. Any\
            lobe for which the maximal peak amplitude is smaller than this\
            threshold will be discarded. Default: 0.1.
        fmls_no_thresholds: disable all FOD lobe thresholding; every lobe where\
            the FOD is positive will be retained.
        fmls_lobe_merge_ratio: Specify the ratio between a given FOD amplitude\
            sample between two lobes, and the smallest peak amplitude of the\
            adjacent lobes, above which those lobes will be merged. This is the\
            amplitude of the FOD at the 'bridge' point between the two lobes,\
            divided by the peak amplitude of the smaller of the two adjoining\
            lobes. A value of 1.0 will never merge two lobes into one; a value of\
            0.0 will always merge lobes unless they are bisected by a zero-valued\
            crossing. Default: 1.
        mask: only perform computation within the specified binary brain mask\
            image.
        maxnum: maximum number of fixels to output for any particular voxel\
            (default: no limit).
        nii: output the directions and index file in nii format (instead of the\
            default mif).
        dirpeak: define the fixel direction as that of the lobe's maximal peak\
            as opposed to its weighted mean direction (the default).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fod2fixel",
        "fmls_no_thresholds": fmls_no_thresholds,
        "nii": nii,
        "dirpeak": dirpeak,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "fod": fod,
        "fixel_directory": fixel_directory,
    }
    if afd is not None:
        params["afd"] = afd
    if peak_amp is not None:
        params["peak_amp"] = peak_amp
    if disp is not None:
        params["disp"] = disp
    if fmls_integral is not None:
        params["fmls_integral"] = fmls_integral
    if fmls_peak_value is not None:
        params["fmls_peak_value"] = fmls_peak_value
    if fmls_lobe_merge_ratio is not None:
        params["fmls_lobe_merge_ratio"] = fmls_lobe_merge_ratio
    if mask is not None:
        params["mask"] = mask
    if maxnum is not None:
        params["maxnum"] = maxnum
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fod2fixel_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fod2fixelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("afd", None) is not None:
        if not isinstance(params["afd"], str):
            raise StyxValidationError(f'`afd` has the wrong type: Received `{type(params.get("afd", None))}` expected `str | None`')
    if params.get("peak_amp", None) is not None:
        if not isinstance(params["peak_amp"], str):
            raise StyxValidationError(f'`peak_amp` has the wrong type: Received `{type(params.get("peak_amp", None))}` expected `str | None`')
    if params.get("disp", None) is not None:
        if not isinstance(params["disp"], str):
            raise StyxValidationError(f'`disp` has the wrong type: Received `{type(params.get("disp", None))}` expected `str | None`')
    if params.get("fmls_integral", None) is not None:
        if not isinstance(params["fmls_integral"], (float, int)):
            raise StyxValidationError(f'`fmls_integral` has the wrong type: Received `{type(params.get("fmls_integral", None))}` expected `float | None`')
    if params.get("fmls_peak_value", None) is not None:
        if not isinstance(params["fmls_peak_value"], (float, int)):
            raise StyxValidationError(f'`fmls_peak_value` has the wrong type: Received `{type(params.get("fmls_peak_value", None))}` expected `float | None`')
    if params.get("fmls_no_thresholds", False) is None:
        raise StyxValidationError("`fmls_no_thresholds` must not be None")
    if not isinstance(params["fmls_no_thresholds"], bool):
        raise StyxValidationError(f'`fmls_no_thresholds` has the wrong type: Received `{type(params.get("fmls_no_thresholds", False))}` expected `bool`')
    if params.get("fmls_lobe_merge_ratio", None) is not None:
        if not isinstance(params["fmls_lobe_merge_ratio"], (float, int)):
            raise StyxValidationError(f'`fmls_lobe_merge_ratio` has the wrong type: Received `{type(params.get("fmls_lobe_merge_ratio", None))}` expected `float | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("maxnum", None) is not None:
        if not isinstance(params["maxnum"], int):
            raise StyxValidationError(f'`maxnum` has the wrong type: Received `{type(params.get("maxnum", None))}` expected `int | None`')
    if params.get("nii", False) is None:
        raise StyxValidationError("`nii` must not be None")
    if not isinstance(params["nii"], bool):
        raise StyxValidationError(f'`nii` has the wrong type: Received `{type(params.get("nii", False))}` expected `bool`')
    if params.get("dirpeak", False) is None:
        raise StyxValidationError("`dirpeak` must not be None")
    if not isinstance(params["dirpeak"], bool):
        raise StyxValidationError(f'`dirpeak` has the wrong type: Received `{type(params.get("dirpeak", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Fod2fixelConfigParamsDict] | None`')
        for e in params["config"]:
            fod2fixel_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("fod", None) is None:
        raise StyxValidationError("`fod` must not be None")
    if not isinstance(params["fod"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fod` has the wrong type: Received `{type(params.get("fod", None))}` expected `InputPathType`')
    if params.get("fixel_directory", None) is None:
        raise StyxValidationError("`fixel_directory` must not be None")
    if not isinstance(params["fixel_directory"], str):
        raise StyxValidationError(f'`fixel_directory` has the wrong type: Received `{type(params.get("fixel_directory", None))}` expected `str`')


def fod2fixel_cargs(
    params: Fod2fixelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fod2fixel")
    if params.get("afd", None) is not None:
        cargs.extend([
            "-afd",
            params.get("afd", None)
        ])
    if params.get("peak_amp", None) is not None:
        cargs.extend([
            "-peak_amp",
            params.get("peak_amp", None)
        ])
    if params.get("disp", None) is not None:
        cargs.extend([
            "-disp",
            params.get("disp", None)
        ])
    if params.get("fmls_integral", None) is not None:
        cargs.extend([
            "-fmls_integral",
            str(params.get("fmls_integral", None))
        ])
    if params.get("fmls_peak_value", None) is not None:
        cargs.extend([
            "-fmls_peak_value",
            str(params.get("fmls_peak_value", None))
        ])
    if params.get("fmls_no_thresholds", False):
        cargs.append("-fmls_no_thresholds")
    if params.get("fmls_lobe_merge_ratio", None) is not None:
        cargs.extend([
            "-fmls_lobe_merge_ratio",
            str(params.get("fmls_lobe_merge_ratio", None))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("maxnum", None) is not None:
        cargs.extend([
            "-maxnum",
            str(params.get("maxnum", None))
        ])
    if params.get("nii", False):
        cargs.append("-nii")
    if params.get("dirpeak", False):
        cargs.append("-dirpeak")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fod2fixel_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("fod", None)))
    cargs.append(params.get("fixel_directory", None))
    return cargs


def fod2fixel_outputs(
    params: Fod2fixelParamsDict,
    execution: Execution,
) -> Fod2fixelOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Fod2fixelOutputs(
        root=execution.output_file("."),
        fixel_directory=execution.output_file(params.get("fixel_directory", None)),
        afd=execution.output_file(params.get("afd", None)) if (params.get("afd") is not None) else None,
        peak_amp=execution.output_file(params.get("peak_amp", None)) if (params.get("peak_amp") is not None) else None,
        disp=execution.output_file(params.get("disp", None)) if (params.get("disp") is not None) else None,
    )
    return ret


def fod2fixel_execute(
    params: Fod2fixelParamsDict,
    runner: Runner | None = None,
) -> Fod2fixelOutputs:
    """
    fod2fixel
    
    Perform segmentation of continuous Fibre Orientation Distributions (FODs) to
    produce discrete fixels.
    
    
    
    References:
    
    * Reference for the FOD segmentation method:
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312 (Appendix 2)
    
    * Reference for Apparent Fibre Density (AFD):
    Raffelt, D.; Tournier, J.-D.; Rose, S.; Ridgway, G.R.; Henderson, R.;
    Crozier, S.; Salvado, O.; Connelly, A. Apparent Fibre Density: a novel
    measure for the analysis of diffusion-weighted magnetic resonance
    images.Neuroimage, 2012, 15;59(4), 3976-94.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fod2fixelOutputs`).
    """
    fod2fixel_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOD2FIXEL_METADATA)
    params = execution.params(params)
    cargs = fod2fixel_cargs(params, execution)
    ret = fod2fixel_outputs(params, execution)
    execution.run(cargs)
    return ret


def fod2fixel(
    fod: InputPathType,
    fixel_directory: str,
    afd: str | None = None,
    peak_amp: str | None = None,
    disp: str | None = None,
    fmls_integral: float | None = None,
    fmls_peak_value: float | None = None,
    fmls_no_thresholds: bool = False,
    fmls_lobe_merge_ratio: float | None = None,
    mask: InputPathType | None = None,
    maxnum: int | None = None,
    nii: bool = False,
    dirpeak: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fod2fixelConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fod2fixelOutputs:
    """
    fod2fixel
    
    Perform segmentation of continuous Fibre Orientation Distributions (FODs) to
    produce discrete fixels.
    
    
    
    References:
    
    * Reference for the FOD segmentation method:
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312 (Appendix 2)
    
    * Reference for Apparent Fibre Density (AFD):
    Raffelt, D.; Tournier, J.-D.; Rose, S.; Ridgway, G.R.; Henderson, R.;
    Crozier, S.; Salvado, O.; Connelly, A. Apparent Fibre Density: a novel
    measure for the analysis of diffusion-weighted magnetic resonance
    images.Neuroimage, 2012, 15;59(4), 3976-94.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        fod: the input fod image.
        fixel_directory: the output fixel directory.
        afd: output the total Apparent Fibre Density per fixel (integral of FOD\
            lobe).
        peak_amp: output the amplitude of the FOD at the maximal peak per fixel.
        disp: output a measure of dispersion per fixel as the ratio between FOD\
            lobe integral and maximal peak amplitude.
        fmls_integral: threshold absolute numerical integral of positive FOD\
            lobes. Any lobe for which the integral is smaller than this threshold\
            will be discarded. Default: 0.
        fmls_peak_value: threshold peak amplitude of positive FOD lobes. Any\
            lobe for which the maximal peak amplitude is smaller than this\
            threshold will be discarded. Default: 0.1.
        fmls_no_thresholds: disable all FOD lobe thresholding; every lobe where\
            the FOD is positive will be retained.
        fmls_lobe_merge_ratio: Specify the ratio between a given FOD amplitude\
            sample between two lobes, and the smallest peak amplitude of the\
            adjacent lobes, above which those lobes will be merged. This is the\
            amplitude of the FOD at the 'bridge' point between the two lobes,\
            divided by the peak amplitude of the smaller of the two adjoining\
            lobes. A value of 1.0 will never merge two lobes into one; a value of\
            0.0 will always merge lobes unless they are bisected by a zero-valued\
            crossing. Default: 1.
        mask: only perform computation within the specified binary brain mask\
            image.
        maxnum: maximum number of fixels to output for any particular voxel\
            (default: no limit).
        nii: output the directions and index file in nii format (instead of the\
            default mif).
        dirpeak: define the fixel direction as that of the lobe's maximal peak\
            as opposed to its weighted mean direction (the default).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fod2fixelOutputs`).
    """
    params = fod2fixel_params(
        afd=afd,
        peak_amp=peak_amp,
        disp=disp,
        fmls_integral=fmls_integral,
        fmls_peak_value=fmls_peak_value,
        fmls_no_thresholds=fmls_no_thresholds,
        fmls_lobe_merge_ratio=fmls_lobe_merge_ratio,
        mask=mask,
        maxnum=maxnum,
        nii=nii,
        dirpeak=dirpeak,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        fod=fod,
        fixel_directory=fixel_directory,
    )
    return fod2fixel_execute(params, runner)


__all__ = [
    "FOD2FIXEL_METADATA",
    "Fod2fixelConfigParamsDict",
    "Fod2fixelConfigParamsDictTagged",
    "Fod2fixelOutputs",
    "Fod2fixelParamsDict",
    "Fod2fixelParamsDictTagged",
    "fod2fixel",
    "fod2fixel_config",
    "fod2fixel_execute",
    "fod2fixel_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRMATH_METADATA = Metadata(
    id="53ebb217f19a2e0a5175c77890d64115b390d7bd.boutiques",
    name="mrmath",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrmathConfigParamsDictNoTag = typing.TypedDict('_MrmathConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrmathConfigParamsDictTagged = typing.TypedDict('MrmathConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrmathConfigParamsDict = _MrmathConfigParamsDictNoTag | MrmathConfigParamsDictTagged


_MrmathParamsDictNoTag = typing.TypedDict('_MrmathParamsDictNoTag', {
    "axis": typing.NotRequired[int | None],
    "keep_unary_axes": bool,
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrmathConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": list[InputPathType],
    "operation": str,
    "output": str,
})
MrmathParamsDictTagged = typing.TypedDict('MrmathParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrmath"],
    "axis": typing.NotRequired[int | None],
    "keep_unary_axes": bool,
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrmathConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": list[InputPathType],
    "operation": str,
    "output": str,
})
MrmathParamsDict = _MrmathParamsDictNoTag | MrmathParamsDictTagged


def mrmath_config(
    key: str,
    value: str,
) -> MrmathConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrmath_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrmathConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrmath_config_cargs(
    params: MrmathConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrmathOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrmathParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""


def mrmath_params(
    input_: list[InputPathType],
    operation: str,
    output: str,
    axis: int | None = None,
    keep_unary_axes: bool = False,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrmathConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrmathParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image(s).
        operation: the operation to apply, one of: mean, median, sum, product,\
            rms, norm, var, std, min, max, absmax, magmax.
        output: the output image.
        axis: perform operation along a specified axis of a single input image.
        keep_unary_axes: Keep unary axes in input images prior to calculating\
            the stats. The default is to wipe axes with single elements.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrmath",
        "keep_unary_axes": keep_unary_axes,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "operation": operation,
        "output": output,
    }
    if axis is not None:
        params["axis"] = axis
    if datatype is not None:
        params["datatype"] = datatype
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrmath_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrmathParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axis", None) is not None:
        if not isinstance(params["axis"], int):
            raise StyxValidationError(f'`axis` has the wrong type: Received `{type(params.get("axis", None))}` expected `int | None`')
    if params.get("keep_unary_axes", False) is None:
        raise StyxValidationError("`keep_unary_axes` must not be None")
    if not isinstance(params["keep_unary_axes"], bool):
        raise StyxValidationError(f'`keep_unary_axes` has the wrong type: Received `{type(params.get("keep_unary_axes", False))}` expected `bool`')
    if params.get("datatype", None) is not None:
        if not isinstance(params["datatype"], str):
            raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrmathConfigParamsDict] | None`')
        for e in params["config"]:
            mrmath_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], list):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `list[InputPathType]`')
    for e in params["input"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `list[InputPathType]`')
    if params.get("operation", None) is None:
        raise StyxValidationError("`operation` must not be None")
    if not isinstance(params["operation"], str):
        raise StyxValidationError(f'`operation` has the wrong type: Received `{type(params.get("operation", None))}` expected `str`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrmath_cargs(
    params: MrmathParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrmath")
    if params.get("axis", None) is not None:
        cargs.extend([
            "-axis",
            str(params.get("axis", None))
        ])
    if params.get("keep_unary_axes", False):
        cargs.append("-keep_unary_axes")
    if params.get("datatype", None) is not None:
        cargs.extend([
            "-datatype",
            params.get("datatype", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrmath_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in params.get("input", None)])
    cargs.append(params.get("operation", None))
    cargs.append(params.get("output", None))
    return cargs


def mrmath_outputs(
    params: MrmathParamsDict,
    execution: Execution,
) -> MrmathOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrmathOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrmath_execute(
    params: MrmathParamsDict,
    runner: Runner | None = None,
) -> MrmathOutputs:
    """
    mrmath
    
    Compute summary statistic on image intensities either across images, or
    along a specified axis of a single image.
    
    Supported operations are:
    
    mean, median, sum, product, rms (root-mean-square value), norm (vector
    2-norm), var (unbiased variance), std (unbiased standard deviation), min,
    max, absmax (maximum absolute value), magmax (value with maximum absolute
    value, preserving its sign).
    
    This command is used to traverse either along an image axis, or across a set
    of input images, calculating some statistic from the values along each
    traversal. If you are seeking to instead perform mathematical calculations
    that are done independently for each voxel, pleaase see the 'mrcalc'
    command.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrmathOutputs`).
    """
    mrmath_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRMATH_METADATA)
    params = execution.params(params)
    cargs = mrmath_cargs(params, execution)
    ret = mrmath_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrmath(
    input_: list[InputPathType],
    operation: str,
    output: str,
    axis: int | None = None,
    keep_unary_axes: bool = False,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrmathConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrmathOutputs:
    """
    mrmath
    
    Compute summary statistic on image intensities either across images, or
    along a specified axis of a single image.
    
    Supported operations are:
    
    mean, median, sum, product, rms (root-mean-square value), norm (vector
    2-norm), var (unbiased variance), std (unbiased standard deviation), min,
    max, absmax (maximum absolute value), magmax (value with maximum absolute
    value, preserving its sign).
    
    This command is used to traverse either along an image axis, or across a set
    of input images, calculating some statistic from the values along each
    traversal. If you are seeking to instead perform mathematical calculations
    that are done independently for each voxel, pleaase see the 'mrcalc'
    command.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image(s).
        operation: the operation to apply, one of: mean, median, sum, product,\
            rms, norm, var, std, min, max, absmax, magmax.
        output: the output image.
        axis: perform operation along a specified axis of a single input image.
        keep_unary_axes: Keep unary axes in input images prior to calculating\
            the stats. The default is to wipe axes with single elements.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrmathOutputs`).
    """
    params = mrmath_params(
        axis=axis,
        keep_unary_axes=keep_unary_axes,
        datatype=datatype,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        operation=operation,
        output=output,
    )
    return mrmath_execute(params, runner)


__all__ = [
    "MRMATH_METADATA",
    "MrmathConfigParamsDict",
    "MrmathConfigParamsDictTagged",
    "MrmathOutputs",
    "MrmathParamsDict",
    "MrmathParamsDictTagged",
    "mrmath",
    "mrmath_config",
    "mrmath_execute",
    "mrmath_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DIRGEN_METADATA = Metadata(
    id="cda69ea8a57022773ddb6022ca8ca9caa5cc0e2b.boutiques",
    name="dirgen",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DirgenConfigParamsDictNoTag = typing.TypedDict('_DirgenConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DirgenConfigParamsDictTagged = typing.TypedDict('DirgenConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DirgenConfigParamsDict = _DirgenConfigParamsDictNoTag | DirgenConfigParamsDictTagged


_DirgenParamsDictNoTag = typing.TypedDict('_DirgenParamsDictNoTag', {
    "power": typing.NotRequired[int | None],
    "niter": typing.NotRequired[int | None],
    "restarts": typing.NotRequired[int | None],
    "unipolar": bool,
    "cartesian": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirgenConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "ndir": int,
    "dirs": str,
})
DirgenParamsDictTagged = typing.TypedDict('DirgenParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dirgen"],
    "power": typing.NotRequired[int | None],
    "niter": typing.NotRequired[int | None],
    "restarts": typing.NotRequired[int | None],
    "unipolar": bool,
    "cartesian": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirgenConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "ndir": int,
    "dirs": str,
})
DirgenParamsDict = _DirgenParamsDictNoTag | DirgenParamsDictTagged


def dirgen_config(
    key: str,
    value: str,
) -> DirgenConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dirgen_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirgenConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dirgen_config_cargs(
    params: DirgenConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DirgenOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DirgenParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    dirs: OutputPathType
    """the text file to write the directions to, as [ az el ] pairs."""


def dirgen_params(
    ndir: int,
    dirs: str,
    power: int | None = None,
    niter: int | None = None,
    restarts: int | None = None,
    unipolar: bool = False,
    cartesian: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirgenConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DirgenParamsDictTagged:
    """
    Build parameters.
    
    Args:
        ndir: the number of directions to generate.
        dirs: the text file to write the directions to, as [ az el ] pairs.
        power: specify exponent to use for repulsion power law (default: 1).\
            This must be a power of 2 (i.e. 1, 2, 4, 8, 16, ...).
        niter: specify the maximum number of iterations to perform (default:\
            10000).
        restarts: specify the number of restarts to perform (default: 10).
        unipolar: optimise assuming a unipolar electrostatic repulsion model\
            rather than the bipolar model normally assumed in DWI.
        cartesian: Output the directions in Cartesian coordinates [x y z]\
            instead of [az el].
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dirgen",
        "unipolar": unipolar,
        "cartesian": cartesian,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "ndir": ndir,
        "dirs": dirs,
    }
    if power is not None:
        params["power"] = power
    if niter is not None:
        params["niter"] = niter
    if restarts is not None:
        params["restarts"] = restarts
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dirgen_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirgenParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("power", None) is not None:
        if not isinstance(params["power"], int):
            raise StyxValidationError(f'`power` has the wrong type: Received `{type(params.get("power", None))}` expected `int | None`')
    if params.get("niter", None) is not None:
        if not isinstance(params["niter"], int):
            raise StyxValidationError(f'`niter` has the wrong type: Received `{type(params.get("niter", None))}` expected `int | None`')
    if params.get("restarts", None) is not None:
        if not isinstance(params["restarts"], int):
            raise StyxValidationError(f'`restarts` has the wrong type: Received `{type(params.get("restarts", None))}` expected `int | None`')
    if params.get("unipolar", False) is None:
        raise StyxValidationError("`unipolar` must not be None")
    if not isinstance(params["unipolar"], bool):
        raise StyxValidationError(f'`unipolar` has the wrong type: Received `{type(params.get("unipolar", False))}` expected `bool`')
    if params.get("cartesian", False) is None:
        raise StyxValidationError("`cartesian` must not be None")
    if not isinstance(params["cartesian"], bool):
        raise StyxValidationError(f'`cartesian` has the wrong type: Received `{type(params.get("cartesian", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DirgenConfigParamsDict] | None`')
        for e in params["config"]:
            dirgen_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("ndir", None) is None:
        raise StyxValidationError("`ndir` must not be None")
    if not isinstance(params["ndir"], int):
        raise StyxValidationError(f'`ndir` has the wrong type: Received `{type(params.get("ndir", None))}` expected `int`')
    if params.get("dirs", None) is None:
        raise StyxValidationError("`dirs` must not be None")
    if not isinstance(params["dirs"], str):
        raise StyxValidationError(f'`dirs` has the wrong type: Received `{type(params.get("dirs", None))}` expected `str`')


def dirgen_cargs(
    params: DirgenParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dirgen")
    if params.get("power", None) is not None:
        cargs.extend([
            "-power",
            str(params.get("power", None))
        ])
    if params.get("niter", None) is not None:
        cargs.extend([
            "-niter",
            str(params.get("niter", None))
        ])
    if params.get("restarts", None) is not None:
        cargs.extend([
            "-restarts",
            str(params.get("restarts", None))
        ])
    if params.get("unipolar", False):
        cargs.append("-unipolar")
    if params.get("cartesian", False):
        cargs.append("-cartesian")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dirgen_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(str(params.get("ndir", None)))
    cargs.append(params.get("dirs", None))
    return cargs


def dirgen_outputs(
    params: DirgenParamsDict,
    execution: Execution,
) -> DirgenOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DirgenOutputs(
        root=execution.output_file("."),
        dirs=execution.output_file(params.get("dirs", None)),
    )
    return ret


def dirgen_execute(
    params: DirgenParamsDict,
    runner: Runner | None = None,
) -> DirgenOutputs:
    """
    dirgen
    
    Generate a set of uniformly distributed directions using a bipolar
    electrostatic repulsion model.
    
    Directions are distributed by analogy to an electrostatic repulsion system,
    with each direction corresponding to a single electrostatic charge (for
    -unipolar), or a pair of diametrically opposed charges (for the default
    bipolar case). The energy of the system is determined based on the Coulomb
    repulsion, which assumes the form 1/r^power, where r is the distance between
    any pair of charges, and p is the power assumed for the repulsion law
    (default: 1). The minimum energy state is obtained by gradient descent.
    
    References:
    
    Jones, D.; Horsfield, M. & Simmons, A. Optimal strategies for measuring
    diffusion in anisotropic systems by magnetic resonance imaging. Magnetic
    Resonance in Medicine, 1999, 42: 515-525
    
    Papadakis, N. G.; Murrills, C. D.; Hall, L. D.; Huang, C. L.-H. & Adrian
    Carpenter, T. Minimal gradient encoding for robust estimation of diffusion
    anisotropy. Magnetic Resonance Imaging, 2000, 18: 671-679.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirgenOutputs`).
    """
    dirgen_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DIRGEN_METADATA)
    params = execution.params(params)
    cargs = dirgen_cargs(params, execution)
    ret = dirgen_outputs(params, execution)
    execution.run(cargs)
    return ret


def dirgen(
    ndir: int,
    dirs: str,
    power: int | None = None,
    niter: int | None = None,
    restarts: int | None = None,
    unipolar: bool = False,
    cartesian: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirgenConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DirgenOutputs:
    """
    dirgen
    
    Generate a set of uniformly distributed directions using a bipolar
    electrostatic repulsion model.
    
    Directions are distributed by analogy to an electrostatic repulsion system,
    with each direction corresponding to a single electrostatic charge (for
    -unipolar), or a pair of diametrically opposed charges (for the default
    bipolar case). The energy of the system is determined based on the Coulomb
    repulsion, which assumes the form 1/r^power, where r is the distance between
    any pair of charges, and p is the power assumed for the repulsion law
    (default: 1). The minimum energy state is obtained by gradient descent.
    
    References:
    
    Jones, D.; Horsfield, M. & Simmons, A. Optimal strategies for measuring
    diffusion in anisotropic systems by magnetic resonance imaging. Magnetic
    Resonance in Medicine, 1999, 42: 515-525
    
    Papadakis, N. G.; Murrills, C. D.; Hall, L. D.; Huang, C. L.-H. & Adrian
    Carpenter, T. Minimal gradient encoding for robust estimation of diffusion
    anisotropy. Magnetic Resonance Imaging, 2000, 18: 671-679.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        ndir: the number of directions to generate.
        dirs: the text file to write the directions to, as [ az el ] pairs.
        power: specify exponent to use for repulsion power law (default: 1).\
            This must be a power of 2 (i.e. 1, 2, 4, 8, 16, ...).
        niter: specify the maximum number of iterations to perform (default:\
            10000).
        restarts: specify the number of restarts to perform (default: 10).
        unipolar: optimise assuming a unipolar electrostatic repulsion model\
            rather than the bipolar model normally assumed in DWI.
        cartesian: Output the directions in Cartesian coordinates [x y z]\
            instead of [az el].
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirgenOutputs`).
    """
    params = dirgen_params(
        power=power,
        niter=niter,
        restarts=restarts,
        unipolar=unipolar,
        cartesian=cartesian,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        ndir=ndir,
        dirs=dirs,
    )
    return dirgen_execute(params, runner)


__all__ = [
    "DIRGEN_METADATA",
    "DirgenConfigParamsDict",
    "DirgenConfigParamsDictTagged",
    "DirgenOutputs",
    "DirgenParamsDict",
    "DirgenParamsDictTagged",
    "dirgen",
    "dirgen_config",
    "dirgen_execute",
    "dirgen_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXELFILTER_METADATA = Metadata(
    id="b5e8a361d7de5f948949260328ab5131eb18a9b7.boutiques",
    name="fixelfilter",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_FixelfilterConfigParamsDictNoTag = typing.TypedDict('_FixelfilterConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
FixelfilterConfigParamsDictTagged = typing.TypedDict('FixelfilterConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
FixelfilterConfigParamsDict = _FixelfilterConfigParamsDictNoTag | FixelfilterConfigParamsDictTagged


_FixelfilterVariousStringParamsDictNoTag = typing.TypedDict('_FixelfilterVariousStringParamsDictNoTag', {
    "obj": str,
})
FixelfilterVariousStringParamsDictTagged = typing.TypedDict('FixelfilterVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
FixelfilterVariousStringParamsDict = _FixelfilterVariousStringParamsDictNoTag | FixelfilterVariousStringParamsDictTagged


_FixelfilterVariousFileParamsDictNoTag = typing.TypedDict('_FixelfilterVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
FixelfilterVariousFileParamsDictTagged = typing.TypedDict('FixelfilterVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
FixelfilterVariousFileParamsDict = _FixelfilterVariousFileParamsDictNoTag | FixelfilterVariousFileParamsDictTagged


_FixelfilterVariousString1ParamsDictNoTag = typing.TypedDict('_FixelfilterVariousString1ParamsDictNoTag', {
    "obj": str,
})
FixelfilterVariousString1ParamsDictTagged = typing.TypedDict('FixelfilterVariousString1ParamsDictTagged', {
    "@type": typing.Literal["VariousString_1"],
    "obj": str,
})
FixelfilterVariousString1ParamsDict = _FixelfilterVariousString1ParamsDictNoTag | FixelfilterVariousString1ParamsDictTagged


_FixelfilterVariousFile1ParamsDictNoTag = typing.TypedDict('_FixelfilterVariousFile1ParamsDictNoTag', {
    "obj": InputPathType,
})
FixelfilterVariousFile1ParamsDictTagged = typing.TypedDict('FixelfilterVariousFile1ParamsDictTagged', {
    "@type": typing.Literal["VariousFile_1"],
    "obj": InputPathType,
})
FixelfilterVariousFile1ParamsDict = _FixelfilterVariousFile1ParamsDictNoTag | FixelfilterVariousFile1ParamsDictTagged


_FixelfilterParamsDictNoTag = typing.TypedDict('_FixelfilterParamsDictNoTag', {
    "matrix": InputPathType,
    "threshold_value": typing.NotRequired[float | None],
    "threshold_connectivity": typing.NotRequired[float | None],
    "fwhm": typing.NotRequired[float | None],
    "minweight": typing.NotRequired[float | None],
    "mask": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelfilterConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": typing.Union[FixelfilterVariousStringParamsDictTagged, FixelfilterVariousFileParamsDictTagged],
    "filter": str,
    "output": typing.Union[FixelfilterVariousString1ParamsDictTagged, FixelfilterVariousFile1ParamsDictTagged],
})
FixelfilterParamsDictTagged = typing.TypedDict('FixelfilterParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fixelfilter"],
    "matrix": InputPathType,
    "threshold_value": typing.NotRequired[float | None],
    "threshold_connectivity": typing.NotRequired[float | None],
    "fwhm": typing.NotRequired[float | None],
    "minweight": typing.NotRequired[float | None],
    "mask": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelfilterConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": typing.Union[FixelfilterVariousStringParamsDictTagged, FixelfilterVariousFileParamsDictTagged],
    "filter": str,
    "output": typing.Union[FixelfilterVariousString1ParamsDictTagged, FixelfilterVariousFile1ParamsDictTagged],
})
FixelfilterParamsDict = _FixelfilterParamsDictNoTag | FixelfilterParamsDictTagged


def fixelfilter_input_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": fixelfilter_various_string_cargs,
        "VariousFile": fixelfilter_various_file_cargs,
    }.get(t)


def fixelfilter_input_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": fixelfilter_various_string_validate,
        "VariousFile": fixelfilter_various_file_validate,
    }.get(t)


def fixelfilter_output_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString_1": fixelfilter_various_string_1_cargs,
        "VariousFile_1": fixelfilter_various_file_1_cargs,
    }.get(t)


def fixelfilter_output_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString_1": fixelfilter_various_string_1_validate,
        "VariousFile_1": fixelfilter_various_file_1_validate,
    }.get(t)


def fixelfilter_config(
    key: str,
    value: str,
) -> FixelfilterConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fixelfilter_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelfilterConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fixelfilter_config_cargs(
    params: FixelfilterConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


def fixelfilter_various_string(
    obj: str,
) -> FixelfilterVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def fixelfilter_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelfilterVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def fixelfilter_various_string_cargs(
    params: FixelfilterVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def fixelfilter_various_file(
    obj: InputPathType,
) -> FixelfilterVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def fixelfilter_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelfilterVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def fixelfilter_various_file_cargs(
    params: FixelfilterVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def fixelfilter_various_string_1(
    obj: str,
) -> FixelfilterVariousString1ParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString_1",
        "obj": obj,
    }
    return params


def fixelfilter_various_string_1_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelfilterVariousString1ParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def fixelfilter_various_string_1_cargs(
    params: FixelfilterVariousString1ParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def fixelfilter_various_file_1(
    obj: InputPathType,
) -> FixelfilterVariousFile1ParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile_1",
        "obj": obj,
    }
    return params


def fixelfilter_various_file_1_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelfilterVariousFile1ParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def fixelfilter_various_file_1_cargs(
    params: FixelfilterVariousFile1ParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


class FixelfilterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FixelfilterParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def fixelfilter_params(
    matrix: InputPathType,
    input_: typing.Union[FixelfilterVariousStringParamsDictTagged, FixelfilterVariousFileParamsDictTagged],
    filter_: str,
    output: typing.Union[FixelfilterVariousString1ParamsDictTagged, FixelfilterVariousFile1ParamsDictTagged],
    threshold_value: float | None = None,
    threshold_connectivity: float | None = None,
    fwhm: float | None = None,
    minweight: float | None = None,
    mask: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelfilterConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> FixelfilterParamsDictTagged:
    """
    Build parameters.
    
    Args:
        matrix: provide a fixel-fixel connectivity matrix for filtering\
            operations that require it.
        input_: the input: either a fixel data file, or a fixel directory (see\
            Description).
        filter_: the filtering operation to perform; options are: connect,\
            smooth.
        output: the output: either a fixel data file, or a fixel directory (see\
            Description).
        threshold_value: specify a threshold for the input fixel data file\
            values (default = 0.5).
        threshold_connectivity: specify a fixel-fixel connectivity threshold\
            for connected-component analysis (default = 0.10000000000000001).
        fwhm: the full-width half-maximum (FWHM) of the spatial component of\
            the smoothing filter (default = 10mm).
        minweight: apply a minimum threshold to smoothing weights (default =\
            0.01).
        mask: only perform smoothing within a specified binary fixel mask.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fixelfilter",
        "matrix": matrix,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "filter": filter_,
        "output": output,
    }
    if threshold_value is not None:
        params["threshold_value"] = threshold_value
    if threshold_connectivity is not None:
        params["threshold_connectivity"] = threshold_connectivity
    if fwhm is not None:
        params["fwhm"] = fwhm
    if minweight is not None:
        params["minweight"] = minweight
    if mask is not None:
        params["mask"] = mask
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixelfilter_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelfilterParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("matrix", None) is None:
        raise StyxValidationError("`matrix` must not be None")
    if not isinstance(params["matrix"], (pathlib.Path, str)):
        raise StyxValidationError(f'`matrix` has the wrong type: Received `{type(params.get("matrix", None))}` expected `InputPathType`')
    if params.get("threshold_value", None) is not None:
        if not isinstance(params["threshold_value"], (float, int)):
            raise StyxValidationError(f'`threshold_value` has the wrong type: Received `{type(params.get("threshold_value", None))}` expected `float | None`')
    if params.get("threshold_connectivity", None) is not None:
        if not isinstance(params["threshold_connectivity"], (float, int)):
            raise StyxValidationError(f'`threshold_connectivity` has the wrong type: Received `{type(params.get("threshold_connectivity", None))}` expected `float | None`')
    if params.get("fwhm", None) is not None:
        if not isinstance(params["fwhm"], (float, int)):
            raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", None))}` expected `float | None`')
    if params.get("minweight", None) is not None:
        if not isinstance(params["minweight"], (float, int)):
            raise StyxValidationError(f'`minweight` has the wrong type: Received `{type(params.get("minweight", None))}` expected `float | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[FixelfilterConfigParamsDict] | None`')
        for e in params["config"]:
            fixelfilter_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["input"])}\'')
    if "@type" not in params["input"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["input"]["@type"] not in ["VariousString", "VariousFile"]:
        raise StyxValidationError("Parameter `input`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
    fixelfilter_input_validate_dyn_fn(params["input"]["@type"])(params["input"])
    if params.get("filter", None) is None:
        raise StyxValidationError("`filter` must not be None")
    if not isinstance(params["filter"], str):
        raise StyxValidationError(f'`filter` has the wrong type: Received `{type(params.get("filter", None))}` expected `str`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["output"])}\'')
    if "@type" not in params["output"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["output"]["@type"] not in ["VariousString_1", "VariousFile_1"]:
        raise StyxValidationError("Parameter `output`s `@type` must be one of [\"VariousString_1\", \"VariousFile_1\"]")
    fixelfilter_output_validate_dyn_fn(params["output"]["@type"])(params["output"])


def fixelfilter_cargs(
    params: FixelfilterParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixelfilter")
    cargs.extend([
        "-matrix",
        execution.input_file(params.get("matrix", None))
    ])
    if params.get("threshold_value", None) is not None:
        cargs.extend([
            "-threshold_value",
            str(params.get("threshold_value", None))
        ])
    if params.get("threshold_connectivity", None) is not None:
        cargs.extend([
            "-threshold_connectivity",
            str(params.get("threshold_connectivity", None))
        ])
    if params.get("fwhm", None) is not None:
        cargs.extend([
            "-fwhm",
            str(params.get("fwhm", None))
        ])
    if params.get("minweight", None) is not None:
        cargs.extend([
            "-minweight",
            str(params.get("minweight", None))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fixelfilter_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend(fixelfilter_input_cargs_dyn_fn(params.get("input", None)["@type"])(params.get("input", None), execution))
    cargs.append(params.get("filter", None))
    cargs.extend(fixelfilter_output_cargs_dyn_fn(params.get("output", None)["@type"])(params.get("output", None), execution))
    return cargs


def fixelfilter_outputs(
    params: FixelfilterParamsDict,
    execution: Execution,
) -> FixelfilterOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FixelfilterOutputs(
        root=execution.output_file("."),
    )
    return ret


def fixelfilter_execute(
    params: FixelfilterParamsDict,
    runner: Runner | None = None,
) -> FixelfilterOutputs:
    """
    fixelfilter
    
    Perform filtering operations on fixel-based data.
    
    If the first input to the command is a specific fixel data file, then a
    filtered version of only that file will be generated by the command.
    Alternatively, if the input is the location of a fixel directory, then the
    command will create a duplicate of the fixel directory, and apply the
    specified filter operation to all fixel data files within the directory.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelfilterOutputs`).
    """
    fixelfilter_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXELFILTER_METADATA)
    params = execution.params(params)
    cargs = fixelfilter_cargs(params, execution)
    ret = fixelfilter_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixelfilter(
    matrix: InputPathType,
    input_: typing.Union[FixelfilterVariousStringParamsDictTagged, FixelfilterVariousFileParamsDictTagged],
    filter_: str,
    output: typing.Union[FixelfilterVariousString1ParamsDictTagged, FixelfilterVariousFile1ParamsDictTagged],
    threshold_value: float | None = None,
    threshold_connectivity: float | None = None,
    fwhm: float | None = None,
    minweight: float | None = None,
    mask: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelfilterConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> FixelfilterOutputs:
    """
    fixelfilter
    
    Perform filtering operations on fixel-based data.
    
    If the first input to the command is a specific fixel data file, then a
    filtered version of only that file will be generated by the command.
    Alternatively, if the input is the location of a fixel directory, then the
    command will create a duplicate of the fixel directory, and apply the
    specified filter operation to all fixel data files within the directory.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        matrix: provide a fixel-fixel connectivity matrix for filtering\
            operations that require it.
        input_: the input: either a fixel data file, or a fixel directory (see\
            Description).
        filter_: the filtering operation to perform; options are: connect,\
            smooth.
        output: the output: either a fixel data file, or a fixel directory (see\
            Description).
        threshold_value: specify a threshold for the input fixel data file\
            values (default = 0.5).
        threshold_connectivity: specify a fixel-fixel connectivity threshold\
            for connected-component analysis (default = 0.10000000000000001).
        fwhm: the full-width half-maximum (FWHM) of the spatial component of\
            the smoothing filter (default = 10mm).
        minweight: apply a minimum threshold to smoothing weights (default =\
            0.01).
        mask: only perform smoothing within a specified binary fixel mask.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelfilterOutputs`).
    """
    params = fixelfilter_params(
        matrix=matrix,
        threshold_value=threshold_value,
        threshold_connectivity=threshold_connectivity,
        fwhm=fwhm,
        minweight=minweight,
        mask=mask,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        filter_=filter_,
        output=output,
    )
    return fixelfilter_execute(params, runner)


__all__ = [
    "FIXELFILTER_METADATA",
    "FixelfilterConfigParamsDict",
    "FixelfilterConfigParamsDictTagged",
    "FixelfilterOutputs",
    "FixelfilterParamsDict",
    "FixelfilterParamsDictTagged",
    "FixelfilterVariousFile1ParamsDict",
    "FixelfilterVariousFile1ParamsDictTagged",
    "FixelfilterVariousFileParamsDict",
    "FixelfilterVariousFileParamsDictTagged",
    "FixelfilterVariousString1ParamsDict",
    "FixelfilterVariousString1ParamsDictTagged",
    "FixelfilterVariousStringParamsDict",
    "FixelfilterVariousStringParamsDictTagged",
    "fixelfilter",
    "fixelfilter_config",
    "fixelfilter_execute",
    "fixelfilter_params",
    "fixelfilter_various_file",
    "fixelfilter_various_file_1",
    "fixelfilter_various_string",
    "fixelfilter_various_string_1",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKDFC_METADATA = Metadata(
    id="c08d7690e556dae567c956d830fbf83e42bf36aa.boutiques",
    name="tckdfc",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TckdfcDynamicParamsDictNoTag = typing.TypedDict('_TckdfcDynamicParamsDictNoTag', {
    "shape": str,
    "width": int,
})
TckdfcDynamicParamsDictTagged = typing.TypedDict('TckdfcDynamicParamsDictTagged', {
    "@type": typing.Literal["dynamic"],
    "shape": str,
    "width": int,
})
TckdfcDynamicParamsDict = _TckdfcDynamicParamsDictNoTag | TckdfcDynamicParamsDictTagged


_TckdfcConfigParamsDictNoTag = typing.TypedDict('_TckdfcConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TckdfcConfigParamsDictTagged = typing.TypedDict('TckdfcConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TckdfcConfigParamsDict = _TckdfcConfigParamsDictNoTag | TckdfcConfigParamsDictTagged


_TckdfcParamsDictNoTag = typing.TypedDict('_TckdfcParamsDictNoTag', {
    "static": bool,
    "dynamic": typing.NotRequired[TckdfcDynamicParamsDict | None],
    "template": typing.NotRequired[InputPathType | None],
    "vox": typing.NotRequired[list[float] | None],
    "stat_vox": typing.NotRequired[str | None],
    "backtrack": bool,
    "upsample": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckdfcConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "fmri": InputPathType,
    "output": str,
})
TckdfcParamsDictTagged = typing.TypedDict('TckdfcParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tckdfc"],
    "static": bool,
    "dynamic": typing.NotRequired[TckdfcDynamicParamsDict | None],
    "template": typing.NotRequired[InputPathType | None],
    "vox": typing.NotRequired[list[float] | None],
    "stat_vox": typing.NotRequired[str | None],
    "backtrack": bool,
    "upsample": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckdfcConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "fmri": InputPathType,
    "output": str,
})
TckdfcParamsDict = _TckdfcParamsDictNoTag | TckdfcParamsDictTagged


def tckdfc_dynamic(
    shape: str,
    width: int,
) -> TckdfcDynamicParamsDictTagged:
    """
    Build parameters.
    
    Args:
        shape: generate a "dynamic" (4D) output image; must additionally\
            provide the shape and width (in volumes) of the sliding window.
        width: generate a "dynamic" (4D) output image; must additionally\
            provide the shape and width (in volumes) of the sliding window.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "dynamic",
        "shape": shape,
        "width": width,
    }
    return params


def tckdfc_dynamic_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckdfcDynamicParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("shape", None) is None:
        raise StyxValidationError("`shape` must not be None")
    if not isinstance(params["shape"], str):
        raise StyxValidationError(f'`shape` has the wrong type: Received `{type(params.get("shape", None))}` expected `str`')
    if params.get("width", None) is None:
        raise StyxValidationError("`width` must not be None")
    if not isinstance(params["width"], int):
        raise StyxValidationError(f'`width` has the wrong type: Received `{type(params.get("width", None))}` expected `int`')


def tckdfc_dynamic_cargs(
    params: TckdfcDynamicParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-dynamic")
    cargs.append(params.get("shape", None))
    cargs.append(str(params.get("width", None)))
    return cargs


def tckdfc_config(
    key: str,
    value: str,
) -> TckdfcConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tckdfc_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckdfcConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tckdfc_config_cargs(
    params: TckdfcConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class TckdfcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TckdfcParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output TW-dFC image"""


def tckdfc_params(
    tracks: InputPathType,
    fmri: InputPathType,
    output: str,
    static: bool = False,
    dynamic: TckdfcDynamicParamsDict | None = None,
    template: InputPathType | None = None,
    vox: list[float] | None = None,
    stat_vox: str | None = None,
    backtrack: bool = False,
    upsample: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckdfcConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TckdfcParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks: the input track file.
        fmri: the pre-processed fMRI time series.
        output: the output TW-dFC image.
        static: generate a "static" (3D) output image.
        dynamic: generate a "dynamic" (4D) output image; must additionally\
            provide the shape and width (in volumes) of the sliding window.
        template: an image file to be used as a template for the output (the\
            output image will have the same transform and field of view).
        vox: provide either an isotropic voxel size (in mm), or comma-separated\
            list of 3 voxel dimensions.
        stat_vox: define the statistic for choosing the final voxel intensities\
            for a given contrast type given the individual values from the tracks\
            passing through each voxel\
            Options are: sum, min, mean, max (default: mean).
        backtrack: if no valid timeseries is found at the streamline endpoint,\
            back-track along the streamline trajectory until a valid timeseries is\
            found.
        upsample: upsample the tracks by some ratio using Hermite interpolation\
            before mapping (if omitted, an appropriate ratio will be determined\
            automatically).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tckdfc",
        "static": static,
        "backtrack": backtrack,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks": tracks,
        "fmri": fmri,
        "output": output,
    }
    if dynamic is not None:
        params["dynamic"] = dynamic
    if template is not None:
        params["template"] = template
    if vox is not None:
        params["vox"] = vox
    if stat_vox is not None:
        params["stat_vox"] = stat_vox
    if upsample is not None:
        params["upsample"] = upsample
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tckdfc_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckdfcParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("static", False) is None:
        raise StyxValidationError("`static` must not be None")
    if not isinstance(params["static"], bool):
        raise StyxValidationError(f'`static` has the wrong type: Received `{type(params.get("static", False))}` expected `bool`')
    if params.get("dynamic", None) is not None:
        tckdfc_dynamic_validate(params["dynamic"])
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("vox", None) is not None:
        if not isinstance(params["vox"], list):
            raise StyxValidationError(f'`vox` has the wrong type: Received `{type(params.get("vox", None))}` expected `list[float] | None`')
        for e in params["vox"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`vox` has the wrong type: Received `{type(params.get("vox", None))}` expected `list[float] | None`')
    if params.get("stat_vox", None) is not None:
        if not isinstance(params["stat_vox"], str):
            raise StyxValidationError(f'`stat_vox` has the wrong type: Received `{type(params.get("stat_vox", None))}` expected `str | None`')
    if params.get("backtrack", False) is None:
        raise StyxValidationError("`backtrack` must not be None")
    if not isinstance(params["backtrack"], bool):
        raise StyxValidationError(f'`backtrack` has the wrong type: Received `{type(params.get("backtrack", False))}` expected `bool`')
    if params.get("upsample", None) is not None:
        if not isinstance(params["upsample"], int):
            raise StyxValidationError(f'`upsample` has the wrong type: Received `{type(params.get("upsample", None))}` expected `int | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TckdfcConfigParamsDict] | None`')
        for e in params["config"]:
            tckdfc_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks", None) is None:
        raise StyxValidationError("`tracks` must not be None")
    if not isinstance(params["tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks` has the wrong type: Received `{type(params.get("tracks", None))}` expected `InputPathType`')
    if params.get("fmri", None) is None:
        raise StyxValidationError("`fmri` must not be None")
    if not isinstance(params["fmri"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fmri` has the wrong type: Received `{type(params.get("fmri", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def tckdfc_cargs(
    params: TckdfcParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tckdfc")
    if params.get("static", False):
        cargs.append("-static")
    if params.get("dynamic", None) is not None:
        cargs.extend(tckdfc_dynamic_cargs(params.get("dynamic", None), execution))
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("vox", None) is not None:
        cargs.extend([
            "-vox",
            ",".join(map(str, params.get("vox", None)))
        ])
    if params.get("stat_vox", None) is not None:
        cargs.extend([
            "-stat_vox",
            params.get("stat_vox", None)
        ])
    if params.get("backtrack", False):
        cargs.append("-backtrack")
    if params.get("upsample", None) is not None:
        cargs.extend([
            "-upsample",
            str(params.get("upsample", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tckdfc_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks", None)))
    cargs.append(execution.input_file(params.get("fmri", None)))
    cargs.append(params.get("output", None))
    return cargs


def tckdfc_outputs(
    params: TckdfcParamsDict,
    execution: Execution,
) -> TckdfcOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TckdfcOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def tckdfc_execute(
    params: TckdfcParamsDict,
    runner: Runner | None = None,
) -> TckdfcOutputs:
    """
    tckdfc
    
    Perform the Track-Weighted Dynamic Functional Connectivity (TW-dFC) method.
    
    This command generates a Track-Weighted Image (TWI), where the contribution
    from each streamline to the image is the Pearson correlation between the
    fMRI time series at the streamline endpoints.
    
    The output image can be generated in one of two ways (note that one of these
    two command-line options MUST be provided):
    
    - "Static" functional connectivity (-static option): Each streamline
    contributes to a static 3D output image based on the correlation between the
    signals at the streamline endpoints using the entirety of the input time
    series.
    
    - "Dynamic" functional connectivity (-dynamic option): The output image is a
    4D image, with the same number of volumes as the input fMRI time series. For
    each volume, the contribution from each streamline is calculated based on a
    finite-width sliding time window, centred at the timepoint corresponding to
    that volume.
    
    Note that the -backtrack option in this command is similar, but not
    precisely equivalent, to back-tracking as can be used with
    Anatomically-Constrained Tractography (ACT) in the tckgen command. However,
    here the feature does not change the streamlines trajectories in any way; it
    simply enables detection of the fact that the input fMRI image may not
    contain a valid timeseries underneath the streamline endpoint, and where
    this occurs, searches from the streamline endpoint inwards along the
    streamline trajectory in search of a valid timeseries to sample from the
    input image.
    
    References:
    
    Calamante, F.; Smith, R.E.; Liang, X.; Zalesky, A.; Connelly, A
    Track-weighted dynamic functional connectivity (TW-dFC): a new method to
    study time-resolved functional connectivity. Brain Struct Funct, 2017, doi:
    10.1007/s00429-017-1431-1.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckdfcOutputs`).
    """
    tckdfc_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKDFC_METADATA)
    params = execution.params(params)
    cargs = tckdfc_cargs(params, execution)
    ret = tckdfc_outputs(params, execution)
    execution.run(cargs)
    return ret


def tckdfc(
    tracks: InputPathType,
    fmri: InputPathType,
    output: str,
    static: bool = False,
    dynamic: TckdfcDynamicParamsDict | None = None,
    template: InputPathType | None = None,
    vox: list[float] | None = None,
    stat_vox: str | None = None,
    backtrack: bool = False,
    upsample: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckdfcConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TckdfcOutputs:
    """
    tckdfc
    
    Perform the Track-Weighted Dynamic Functional Connectivity (TW-dFC) method.
    
    This command generates a Track-Weighted Image (TWI), where the contribution
    from each streamline to the image is the Pearson correlation between the
    fMRI time series at the streamline endpoints.
    
    The output image can be generated in one of two ways (note that one of these
    two command-line options MUST be provided):
    
    - "Static" functional connectivity (-static option): Each streamline
    contributes to a static 3D output image based on the correlation between the
    signals at the streamline endpoints using the entirety of the input time
    series.
    
    - "Dynamic" functional connectivity (-dynamic option): The output image is a
    4D image, with the same number of volumes as the input fMRI time series. For
    each volume, the contribution from each streamline is calculated based on a
    finite-width sliding time window, centred at the timepoint corresponding to
    that volume.
    
    Note that the -backtrack option in this command is similar, but not
    precisely equivalent, to back-tracking as can be used with
    Anatomically-Constrained Tractography (ACT) in the tckgen command. However,
    here the feature does not change the streamlines trajectories in any way; it
    simply enables detection of the fact that the input fMRI image may not
    contain a valid timeseries underneath the streamline endpoint, and where
    this occurs, searches from the streamline endpoint inwards along the
    streamline trajectory in search of a valid timeseries to sample from the
    input image.
    
    References:
    
    Calamante, F.; Smith, R.E.; Liang, X.; Zalesky, A.; Connelly, A
    Track-weighted dynamic functional connectivity (TW-dFC): a new method to
    study time-resolved functional connectivity. Brain Struct Funct, 2017, doi:
    10.1007/s00429-017-1431-1.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks: the input track file.
        fmri: the pre-processed fMRI time series.
        output: the output TW-dFC image.
        static: generate a "static" (3D) output image.
        dynamic: generate a "dynamic" (4D) output image; must additionally\
            provide the shape and width (in volumes) of the sliding window.
        template: an image file to be used as a template for the output (the\
            output image will have the same transform and field of view).
        vox: provide either an isotropic voxel size (in mm), or comma-separated\
            list of 3 voxel dimensions.
        stat_vox: define the statistic for choosing the final voxel intensities\
            for a given contrast type given the individual values from the tracks\
            passing through each voxel\
            Options are: sum, min, mean, max (default: mean).
        backtrack: if no valid timeseries is found at the streamline endpoint,\
            back-track along the streamline trajectory until a valid timeseries is\
            found.
        upsample: upsample the tracks by some ratio using Hermite interpolation\
            before mapping (if omitted, an appropriate ratio will be determined\
            automatically).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckdfcOutputs`).
    """
    params = tckdfc_params(
        static=static,
        dynamic=dynamic,
        template=template,
        vox=vox,
        stat_vox=stat_vox,
        backtrack=backtrack,
        upsample=upsample,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks=tracks,
        fmri=fmri,
        output=output,
    )
    return tckdfc_execute(params, runner)


__all__ = [
    "TCKDFC_METADATA",
    "TckdfcConfigParamsDict",
    "TckdfcConfigParamsDictTagged",
    "TckdfcDynamicParamsDict",
    "TckdfcDynamicParamsDictTagged",
    "TckdfcOutputs",
    "TckdfcParamsDict",
    "TckdfcParamsDictTagged",
    "tckdfc",
    "tckdfc_config",
    "tckdfc_dynamic",
    "tckdfc_execute",
    "tckdfc_params",
]
